from typing import Dict, List, Optional, Callable
from enum import Enum

from .pyobject import Segment, Credit


def download(url: str,
             header_map: Dict[str, str],
             file_name: str,
             segment: Segment,
             proxy: Optional[str]) -> None:
    """
    下载视频

    :param str url: 视频地址
    :param Dict[str, str] header_map: HTTP请求头
    :param str file_name: 文件名格式
    :param Segment segment: 视频分段设置
    :param Optional[str] proxy: 代理
    """


def download_with_callback(url: str,
               header_map: Dict[str, str],
               file_name: str,
               segment: Segment,
               file_name_callback_fn: Callable[[str], None],
               proxy: Optional[str]) -> None:
    """
    下载视频

    :param str url: 视频地址
    :param Dict[str, str] header_map: HTTP请求头
    :param str file_name: 文件名格式
    :param Segment segment: 视频分段设置
    :param Callable[[str], None] file_name_callback_fn: 回调已下载完成文件名
    :param Optional[str] proxy: 代理
    """


def login_by_cookies(proxy: Optional[str]) -> bool:
    """
    cookie登录

    :param Optional[str] proxy: 代理
    :return: 是否登录成功
    """


def send_sms(country_code: int, phone: int, proxy: Optional[str]) -> str:
    """
    发送短信验证码

    :param int country_code: 国家/地区代码
    :param int phone: 手机号
    :param Optional[str] proxy: 代理
    :return: 短信登录JSON信息
    """


def login_by_sms(code: int, ret: str, proxy: Optional[str]) -> bool:
    """
    短信登录

    :param int code: 验证码
    :param str ret: 短信登录JSON信息
    :param Optional[str] proxy: 代理
    :return: 是否登录成功
    """


def get_qrcode(proxy: Optional[str]) -> str:
    """
    获取二维码

    :param Optional[str] proxy: 代理
    :return: 二维码登录JSON信息
    """


def login_by_qrcode(ret: str, proxy: Optional[str]) -> bool:
    """
    二维码登录

    :param str ret: 二维码登录JSON信息
    :param Optional[str] proxy: 代理
    :return: 是否登录成功
    """


def login_by_web_cookies(sess_data: str, bili_jct: str, proxy: Optional[str]) -> bool:
    """
    网页Cookie登录1

    :param str sess_data: SESSDATA
    :param str bili_jct: bili_jct
    :param Optional[str] proxy: 代理
    :return: 是否登录成功
    """


def login_by_web_qrcode(sess_data: str, dede_user_id: str, proxy: Optional[str]) -> bool:
    """
    网页Cookie登录2

    :param str sess_data: SESSDATA
    :param str dede_user_id: DedeUserID
    :param Optional[str] proxy: 代理
    :return: 是否登录成功
    """


class UploadLine(Enum):
    """上传线路"""

    Bda2 = 1
    """百度云"""

    Qn = 2
    """七牛"""

    Bda = 3
    """百度云海外"""

    Tx = 4
    """腾讯云EO"""

    Txa = 5
    """腾讯云EO海外"""

    Bldsa = 6
    """Bilibili大陆动态加速"""

    Alia = 7
    """阿里云海外"""


def upload(video_path: List[str],
           cookie_file: str,
           title: str,
           tid: int,
           tag: str,
           copyright: int,
           source: str,
           desc: str,
           dynamic: str,
           cover: str,
           dolby: int,
           lossless_music: int,
           no_reprint: int,
           charging_pay: int,
           limit: int,
           desc_v2: List[Credit],
           dtime: Optional[int],
           line: Optional[UploadLine],
           extra_fields: Optional[str],
           submit: Optional[str],
           proxy: Optional[str]) -> None:

    """
    上传视频稿件

    :param List[str] video_path: 视频文件路径
    :param str cookie_file: cookie文件路径
    :param str title: 视频标题
    :param int tid: 投稿分区
    :param str tag: 视频标签, 英文逗号分隔多个tag
    :param int copyright: 是否转载, 1-自制 2-转载
    :param str source: 转载来源
    :param str desc: 视频简介
    :param str dynamic: 空间动态
    :param str cover: 视频封面
    :param int dolby: 是否开启杜比音效, 0-关闭 1-开启
    :param int lossless_music: 是否开启Hi-Res, 0-关闭 1-开启
    :param int no_reprint: 是否禁止转载, 0-允许 1-禁止
    :param int charging_pay: 是否开启充电, 0-关闭 1-开启
    :param int limit: 单视频文件最大并发数
    :param List[Credit] desc_v2: 视频简介v2
    :param Optional[dtime] int dtime: 定时发布时间, 距离提交大于2小时小于15天, 格式为10位时间戳
    :param Optional[UploadLine] line: 上传线路
    :param Optional[ExtraFields] line: 上传额外参数
    :param Optional[str] submit: 提交接口, 可选值: BCutAndroid, App（默认）
    :param Optional[str] proxy: 代理
    """



import re
from typing import AsyncGenerator, List

from . import logger
from ..common.util import client
from ..engine.decorators import Plugin
from ..engine.download import DownloadBase, BatchCheck

VALID_URL_BASE = r"(?:https?://)?(?:www\.)?picarto\.tv/(?P<id>[^/?&]+)"
API_CHANNEL = "https://ptvintern.picarto.tv/api/channel/detail/{username}"
API_EXPLORE = "https://ptvintern.picarto.tv/api/explore?first=100&page={page}&filter_params%5Badult%5D=true&order_by%5Bfield%5D=viewers&order_by%5Border%5D=DESC&type=stream"
CHANNEL_URL = "https://picarto.tv/{user_name}"
HLS_URL = "https://{netloc}.picarto.tv/stream/hls/{file_name}/index.m3u8"

@Plugin.download(regexp=VALID_URL_BASE)
class Picarto(DownloadBase, BatchCheck):

    def __init__(self, fname, url, suffix="flv"):
        super().__init__(fname, url, suffix)

    async def acheck_stream(self, is_check=False):
        username = re.match(VALID_URL_BASE, self.url).group("id")
        channel_detail = (await client.get(
            API_CHANNEL.format(username=username), timeout=5
        )).json()
        channel = channel_detail.get("channel", {})
        loadbalancer = channel_detail.get("getLoadBalancerUrl", {})
        multistreams = channel_detail.get("getMultiStreams", {})

        # 檢查response
        if not channel or not multistreams or not loadbalancer:
            return False
        elif channel.get("private"):
            logger.warning("This is a private stream")
            return False

        user_id = channel.get("id")
        if not (
            stream := next(
                (
                    stream
                    for stream in multistreams.get("streams")
                    if stream.get("channelId") == user_id
                ),
                None,
            )
        ):
            logger.warning("No available stream found in 'multistreams' data")
            return False

        self.room_title = channel.get("title")
        self.live_cover_url = stream.get("thumbnail_image")
        if is_check:
            return True

        self.raw_stream_url = HLS_URL.format(
            netloc=loadbalancer.get("origin"), file_name=stream.get("stream_name")
        )
        return True

    @staticmethod
    async def abatch_check(check_urls: List[str]) -> AsyncGenerator[str, None]:
        explore = []
        page = 1

        while True:
            explore_detail = (await client.get(API_EXPLORE.format(page=page), timeout=5)).json()
            explore += explore_detail.get("data", [])
            if explore_detail.get("next_page_url"):
                page += 1
            else:
                break

        for data in explore:
            if (
                channel := CHANNEL_URL.format(user_name=data.get("name"))
            ) in check_urls:
                yield channel



import struct

from biliup.common.tars.__tup import TarsUniPacket
from biliup.common.tars.__packet import RequestPacket
from biliup.common.tars.__tars import (TarsInputStream, TarsOutputStream)
from biliup.common.tars.__util import util

from .packet.__util import STANDARD_CHARSET


class Wup(TarsUniPacket):
    def __init__(self):
        super().__init__()
        self.__mapa = util.mapclass(util.string, util.bytes)
        self.__new_buffer = self.__mapa()
        self.__code = RequestPacket()

    @property
    def version(self):
        return self.__code.iVersion

    @version.setter
    def version(self, value):
        self.__code.iVersion = value

    @property
    def servant(self):
        return self.__code.sServantName

    @servant.setter
    def servant(self, value):
        self.__code.sServantName = value

    @property
    def func(self):
        return self.__code.sFuncName

    @func.setter
    def func(self, value):
        self.__code.sFuncName = value

    @property
    def requestid(self):
        return self.__code.iRequestId

    @requestid.setter
    def requestid(self, value):
        self.__code.iRequestId = value

    def put(self, vtype, name, value):
        oos = TarsOutputStream()
        oos.write(vtype, 0, value)
        self.__new_buffer[name] = oos.getBuffer()

    def get(self, vtype, name):
        if isinstance(name, str):
            name = name.encode(STANDARD_CHARSET)

        if (name in self.__new_buffer) == False:
            raise Exception("UniAttribute not found key:%s" % name)

        t = self.__new_buffer[name]
        o = TarsInputStream(t)
        return o.read(vtype, 0, True)

    def encode_v3(self):
        oos = TarsOutputStream()
        oos.write(self.__mapa, 0, self.__new_buffer)

        self.__code.iVersion = 3 # Force to use TarsV3
        self.__code.sBuffer = oos.getBuffer()

        sos = TarsOutputStream()
        RequestPacket.writeTo(sos, self.__code)

        return struct.pack('!i', 4 + len(sos.getBuffer())) + sos.getBuffer()

    def decode_v3(self, buf):
        ois = TarsInputStream(buf[4:])
        self.__code = RequestPacket.readFrom(ois)

        sis = TarsInputStream(self.__code.sBuffer)
        self.__new_buffer = sis.read(self.__mapa, 0, True)

    def clear(self):
        self.__code.__init__()